<?php

namespace Drupal\search_api_japanese_normalizer\Utility;

/**
 * Provides a Utility Japanese conversion class based on NEologd.
 *
 * This class is a utility class used when normalizing Japanese.
 */
class NeologdNormalizer {

  /**
   * A constant that defines a Japanese character class.
   *
   * This constant is used in regular expressions to match Japanese characters
   * Hiragana katakana, kanji.
   *
   * `\p{Hiragana}` : Hiragana characters
   * `\p{Katakana}` : Katakana characters
   *  \p{Han}`      : Kanji characters
   *
   *  @see https://so-zou.jp/software/tech/programming/tech/regular-expression/meta-character/variable-width-encoding.htm
   */
  const PREG_JAPANESE_CLASS = '\p{Hiragana}\p{Katakana}\p{Han}！-～';

  /**
   * Normalize all strings.
   *
   * @param string $value
   *   The input string to be normalized.
   *
   * @return string
   *   String with all transformations applied.
   */
  public static function normalize(string $value = ''): string {
    $methods = [
      'convertFullWidthAlphanumeric',
      'convertHalfToFullKana',
      'convertHyphens',
      'convertLongHyphens',
      'convertLongVowelSymbols',
      'convertDeleteTildeSymbols',
      'convertHalfWidthSymbols',
      'convertHalfWidthSymbolsToFullWidth',
      'convertFullWidthSpaceToHalfWidth',
      'convertMultipleSpaces',
      'convertUnnecessaryHalfWidthSpaces',
      'convertTrimSpaceJapaneseAlpha',
    ];

    foreach ($methods as $method) {
      $value = self::$method($value);
    }

    return $value;
  }

  /**
   * Replace full-width alphanumeric characters with half-width ones.
   *
   * @param string $input
   *   The input string.
   *
   * @return string
   *   The converted string with half-width alphanumeric characters.
   *
   * @see https://github.com/neologd/mecab-ipadic-neologd/wiki/Regexp.ja#%E5%85%A8%E8%A7%92%E8%8B%B1%E6%95%B0%E5%AD%97%E3%81%AF%E5%8D%8A%E8%A7%92%E3%81%AB%E7%BD%AE%E6%8F%9B
   */
  public static function convertFullWidthAlphanumeric(string $input = ''): string {
    return mb_convert_kana($input, 'rn');
  }

  /**
   * Replace half-width katakana with full-width.
   *
   * @param string $input
   *   The input string.
   *
   * @return string
   *   The converted string with half-width katakana with full-width.
   *
   * @see https://github.com/neologd/mecab-ipadic-neologd/wiki/Regexp.ja#%E5%8D%8A%E8%A7%92%E3%82%AB%E3%82%BF%E3%82%AB%E3%83%8A%E3%81%AF%E5%85%A8%E8%A7%92%E3%81%AB%E7%BD%AE%E6%8F%9B
   */
  public static function convertHalfToFullKana(string $input = ''): string {
    return mb_convert_kana($input, 'KV');
  }

  /**
   * Replace hyphen-minus-like characters.
   *
   * @param string $input
   *   The input string.
   *
   * @return string
   *   The hyphen-minus-like characters.
   *
   * @see https://symbl.cc/
   * @see https://github.com/neologd/mecab-ipadic-neologd/wiki/Regexp.ja#%E3%83%8F%E3%82%A4%E3%83%95%E3%83%B3%E3%83%9E%E3%82%A4%E3%83%8A%E3%82%B9%E3%81%A3%E3%81%BD%E3%81%84%E6%96%87%E5%AD%97%E3%82%92%E7%BD%AE%E6%8F%9B
   */
  public static function convertHyphens(string $input = ''): string {
    $pattern = '/[˗֊‐‑‒–⁃⁻₋−]/u';
    return preg_replace($pattern, '-', $input);
  }

  /**
   * Replace long-syllable-like characters.
   *
   * @param string $input
   *   The input string.
   *
   * @return string
   *   The converted string with long-syllable-like characters.
   *
   * @see https://symbl.cc/
   * @see https://github.com/neologd/mecab-ipadic-neologd/wiki/Regexp.ja#%E9%95%B7%E9%9F%B3%E8%A8%98%E5%8F%B7%E3%81%A3%E3%81%BD%E3%81%84%E6%96%87%E5%AD%97%E3%82%92%E7%BD%AE%E6%8F%9B
   */
  public static function convertLongHyphens(string $input = ''): string {
    $patterns = '/[—―─━﹣－ｰ]/u';
    return preg_replace($patterns, 'ー', $input);
  }

  /**
   * More than one consecutive long vowel symbol is replaced by one.
   *
   * @see https://www.megasoft.co.jp/mifes/seiki/
   * @see https://github.com/neologd/mecab-ipadic-neologd/wiki/Regexp.ja#1%E5%9B%9E%E4%BB%A5%E4%B8%8A%E9%80%A3%E7%B6%9A%E3%81%99%E3%82%8B%E9%95%B7%E9%9F%B3%E8%A8%98%E5%8F%B7%E3%81%AF1%E5%9B%9E%E3%81%AB%E7%BD%AE%E6%8F%9B
   */
  public static function convertLongVowelSymbols(string $input = ''): string {
    return preg_replace('/ー{2,}/u', 'ー', $input);
  }

  /**
   * Delete tilde-like characters.
   *
   * @param string $input
   *   The input string.
   *
   * @return string
   *   The converted string with Delete tilde-like characters.
   *
   * @see https://github.com/neologd/mecab-ipadic-neologd/wiki/Regexp.ja#%E3%83%81%E3%83%AB%E3%83%80%E3%81%A3%E3%81%BD%E3%81%84%E6%96%87%E5%AD%97%E3%81%AF%E5%89%8A%E9%99%A4
   */
  public static function convertDeleteTildeSymbols(string $input = ''): string {
    $patterns = '/[～∼∾〜〰～]/u';
    return preg_replace($patterns, '', $input);
  }

  /**
   * Replace full-width symbols with half-width symbols.
   *
   * @param string $input
   *   The input string.
   *
   * @return string
   *   The converted string with full-width symbols with half-width symbols.
   *
   * @see https://github.com/neologd/mecab-ipadic-neologd/wiki/Regexp.ja#%E4%BB%A5%E4%B8%8B%E3%81%AE%E5%85%A8%E8%A7%92%E8%A8%98%E5%8F%B7%E3%81%AF%E5%8D%8A%E8%A7%92%E8%A8%98%E5%8F%B7%E3%81%AB%E7%BD%AE%E6%8F%9B
   */
  public static function convertHalfWidthSymbols(string $input = ''): string {
    $table = [
      '／' => '/',
      '！' => '!',
      '”' => '”',
      '＃' => '#',
      '＄' => '$',
      '％' => '%',
      '＆' => '&',
      '’' => '’',
      '（' => '(',
      '）' => ')',
      '＊' => '*',
      '＋' => '+',
      '，' => ',',
      '−' => '-',
      '．' => '.',
      '：' => ':',
      '；' => ';',
      '＜' => '<',
      '＞' => '>',
      '？' => '?',
      '＠' => '@',
      '［' => '[',
      '￥' => '¥',
      '］' => ']',
      '＾' => '^',
      '＿' => '_',
      '｀' => '`',
      '｛' => '{',
      '｜' => '|',
      '｝' => '}',
    ];

    $search = array_keys($table);
    $replace = array_values($table);
    return str_replace($search, $replace, $input);
  }

  /**
   * Half-width symbols are replaced with full-width symbols.
   *
   * @param string $input
   *   The input string.
   *
   * @return string
   *   The converted string symbols are replaced with full-width symbols.
   *
   * @see https://github.com/neologd/mecab-ipadic-neologd/wiki/Regexp.ja#%E4%BB%A5%E4%B8%8B%E3%81%AE%E5%8D%8A%E8%A7%92%E8%A8%98%E5%8F%B7%E3%81%AF%E5%85%A8%E8%A7%92%E8%A8%98%E5%8F%B7%E3%81%AB%E7%BD%AE%E6%8F%9B
   */
  public static function convertHalfWidthSymbolsToFullWidth(string $input = ''): string {
    $table = [
      '｡' => '。',
      '､' => '、',
      '･' => '・',
      '=' => '＝',
      '｢' => '「',
      '｣' => '」',
    ];

    $search = array_keys($table);
    $replace = array_values($table);
    return str_replace($search, $replace, $input);
  }

  /**
   * Replace full-width spaces with half-width spaces.
   *
   * @param string $input
   *   The input string.
   *
   * @return string
   *   The converted string half-width spaces in converted strings.
   *
   * @see https://github.com/neologd/mecab-ipadic-neologd/wiki/Regexp.ja#%E5%85%A8%E8%A7%92%E3%82%B9%E3%83%9A%E3%83%BC%E3%82%B9%E3%81%AF%E5%8D%8A%E8%A7%92%E3%82%B9%E3%83%9A%E3%83%BC%E3%82%B9%E3%81%AB%E7%BD%AE%E6%8F%9B
   */
  public static function convertFullWidthSpaceToHalfWidth(string $input = ''): string {
    return str_replace('　', ' ', $input);
  }

  /**
   * Replace one or more half-width spaces with one half-width space.
   *
   * @param string $input
   *   The input string.
   *
   * @return string
   *   The converted string 1 half-width space.
   *
   * @see https://github.com/neologd/mecab-ipadic-neologd/wiki/Regexp.ja#1%E3%81%A4%E4%BB%A5%E4%B8%8A%E3%81%AE%E5%8D%8A%E8%A7%92%E3%82%B9%E3%83%9A%E3%83%BC%E3%82%B9%E3%81%AF1%E3%81%A4%E3%81%AE%E5%8D%8A%E8%A7%92%E3%82%B9%E3%83%9A%E3%83%BC%E3%82%B9%E3%81%AB%E7%BD%AE%E6%8F%9B
   */
  public static function convertMultipleSpaces(string $input = ''): string {
    return preg_replace('/\s+/', ' ', $input);
  }

  /**
   * Delete half-width spaces between specific character types.
   *
   * Delete half-width spaces between "hiragana, full-width katakana,
   * half-width katakana, kanji, and full-width symbols.
   *
   * @param string $input
   *   The input string.
   *
   * @return string
   *   The converted string Delete half-width spaces.
   *
   * @see https://github.com/neologd/mecab-ipadic-neologd/wiki/Regexp.ja#%E3%81%B2%E3%82%89%E3%81%8C%E3%81%AA%E5%85%A8%E8%A7%92%E3%82%AB%E3%82%BF%E3%82%AB%E3%83%8A%E5%8D%8A%E8%A7%92%E3%82%AB%E3%82%BF%E3%82%AB%E3%83%8A%E6%BC%A2%E5%AD%97%E5%85%A8%E8%A7%92%E8%A8%98%E5%8F%B7%E9%96%93%E3%81%AB%E5%90%AB%E3%81%BE%E3%82%8C%E3%82%8B%E5%8D%8A%E8%A7%92%E3%82%B9%E3%83%9A%E3%83%BC%E3%82%B9%E3%81%AF%E5%89%8A%E9%99%A4
   */
  public static function convertUnnecessaryHalfWidthSpaces(string $input = ''): string {
    $pattern = '/(?<=[' . self::PREG_JAPANESE_CLASS .
    '])\s+(?=[' .
    self::PREG_JAPANESE_CLASS . '])/u';
    return preg_replace($pattern, '', $input);
  }

  /**
   * Delete half-width spaces between specific character types.
   *
   * Delete half-width spaces between "hiragana, full-width katakana,
   * half-width katakana, kanji, full-width symbols" and "half-width alphanumeric characters".
   *
   * @param string $input
   *   The input string.
   *
   * @return string
   *   The converted string Delete half-width spaces.
   *
   * @see https://github.com/neologd/mecab-ipadic-neologd/wiki/Regexp.ja#%E3%81%B2%E3%82%89%E3%81%8C%E3%81%AA%E5%85%A8%E8%A7%92%E3%82%AB%E3%82%BF%E3%82%AB%E3%83%8A%E5%8D%8A%E8%A7%92%E3%82%AB%E3%82%BF%E3%82%AB%E3%83%8A%E6%BC%A2%E5%AD%97%E5%85%A8%E8%A7%92%E8%A8%98%E5%8F%B7%E3%81%A8%E5%8D%8A%E8%A7%92%E8%8B%B1%E6%95%B0%E5%AD%97%E3%81%AE%E9%96%93%E3%81%AB%E5%90%AB%E3%81%BE%E3%82%8C%E3%82%8B%E5%8D%8A%E8%A7%92%E3%82%B9%E3%83%9A%E3%83%BC%E3%82%B9%E3%81%AF%E5%89%8A%E9%99%A4
   */
  public static function convertTrimSpaceJapaneseAlpha(string $input): string {
    $pattern = '/(?<=[' .
    self::PREG_JAPANESE_CLASS .
    '])\s+(?=[' .
    self::PREG_JAPANESE_CLASS . 'A-Za-z0-9])|(?<=[A-Za-z0-9])\s+(?=['
    . self::PREG_JAPANESE_CLASS . '])/u';
    return preg_replace($pattern, '', $input);
  }

}
