<?php

namespace Drupal\Tests\search_api_japanese_normalizer\Unit\Utility;

use Drupal\search_api_japanese_normalizer\Utility\NeologdNormalizer;
use Drupal\Tests\UnitTestCase;

/**
 * Unit tests for the NeologdNormalizer utility class.
 *
 * @group search_api_japanese_normalizer
 */
class NeologdNormalizerTest extends UnitTestCase {

  /**
   * Tests normalize method.
   *
   * @dataProvider provideNormalize
   */
  public function testNormalize($input, $expected) {
    $this->assertEquals($expected, NeologdNormalizer::normalize($input));
  }

  /**
   * Data provider for testNormalize.
   */
  public function provideNormalize() {
    return [
      // Replace full-width alphanumeric characters with half-width.
      [
        'ＡＢＣＤＥＦＧＨＩＪＫＬＭＮＯＰＱＲＳＴＵＶＷＸＹＺａｂｃｄｅｆｇｈｉｊｋｌｍｎｏｐｑｒｓｔｕｖｗｘｙｚ０１２３４５６７８９',
        'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789',
      ],
      // Replace full-width Katakana with full-width Hiragana.
      ['ｱｲｳｴｵｶｷｸｹｺｻｼｽｾｿﾀﾁﾂﾃﾄﾅﾆﾇﾈﾉﾊﾋﾌﾍﾎﾏﾐﾑﾒﾓﾔﾕﾖﾗﾘﾙﾚﾛﾜヰヱｦﾝ', 'アイウエオカキクケコサシスセソタチツテトナニヌネノハヒフヘホマミムメモヤユヨラリルレロワヰヱヲン'],
      // Replace characters resembling hyphens or minus signs.
      ['˗֊‐‑‒–⁃⁻₋−', '----------'],
      // Replace characters resembling long vowel marks.
      ['—|―|─|━|﹣|－|ｰ', 'ー|ー|ー|ー|ー|ー|ー'],
      // Replace one or more consecutive long vowel marks with a single one.
      ['スーーーーーーパーーーーーー', 'スーパー'],
      // Remove characters resembling tildes.
      ['～∼∾〜〰～', ''],
      // Replace the following full-width symbols with half-width ones.
      ['／！”＃＄％＆’（）＊＋，−．／：；＜＞？＠［￥］＾＿｀｛｜｝', '/!”#$%&’()*+,-./:;<>?@[¥]^_`{|}'],
      // Replace the following half-width symbols with full-width ones.
      ['｡､･=｢｣', '。、・＝「」'],
      // Replace full-width spaces with half-width spaces.
      ['Hello World', 'Hello World'],
      // Replace one or more consecutive half-width spaces with a single space.
      ['Hello    World', 'Hello World'],
      // Remove half-width spaces between Hiragana, full-width Katakana,
      // half-width Katakana, Kanji, and full-width symbols.
      ['検索 エンジン 自作 入門 を 買い ました', '検索エンジン自作入門を買いました'],
      ['Coding the Matrix', 'Coding the Matrix'],
      // Remove half-width spaces between Japanese characters (Hiragana,
      // Katakana, Kanji, full-width symbols) and alphanumeric characters.
      ['アルゴリズム C', 'アルゴリズムC'],
      ['Algorithm C', 'Algorithm C'],
      // Composite test for mixing full-width and half-width alphanumeric
      // characters.
      ['最強の ＣＭＳ  Drupal', '最強のCMS Drupal'],
      // Composite test for full-width English conversion and space removal.
      ['Ｔｈｅ  ｇｒｅａｔｅｓｔ  ＣＭＳ  Ｄｒｕｐａｌ', 'The greatest CMS Drupal'],
      // Composite test for consecutive long vowel marks with various
      // characters resembling them.
      ['ス—―─━ー﹣－ｰパー—―─━﹣－ｰー', 'スーパー'],
      // Composite test for consecutive full-width and half-width spaces in a
      // Japanese sentence.
      ['明   鏡  止   水', '明鏡止水'],
      // Composite test for full-width and half-width spaces in English within
      // a Japanese sentence.
      ['これは Very   nice   CMS です', 'これはVery nice CMSです'],
    ];
  }

  /**
   * Tests convertFullWidthAlphanumeric method.
   */
  public function testConvertFullWidthAlphanumeric() {
    $input = 'ＡＢＣ１２３';
    $expected = 'ABC123';
    $this->assertEquals($expected, NeologdNormalizer::convertFullWidthAlphanumeric($input));
  }

  /**
   * Tests convertHalfToFullKana method.
   */
  public function testConvertHalfToFullKana() {
    $input = 'ｱｲｳｴｵ';
    $expected = 'アイウエオ';
    $this->assertEquals($expected, NeologdNormalizer::convertHalfToFullKana($input));
  }

  /**
   * Tests convertHyphens method.
   */
  public function testConvertHyphens() {
    $input = '˗֊‐‑‒–⁃⁻₋−';
    $expected = '----------';
    $this->assertEquals($expected, NeologdNormalizer::convertHyphens($input));
  }

  /**
   * Tests convertLongHyphens method.
   */
  public function testConvertLongHyphens() {
    $input = '—―─━﹣－ｰ';
    $expected = 'ーーーーーーー';
    $this->assertEquals($expected, NeologdNormalizer::convertLongHyphens($input));
  }

  /**
   * Tests convertLongVowelSymbols method.
   */
  public function testConvertLongVowelSymbols() {
    $input = 'スーーーーーーパーーーーーー';
    $expected = 'スーパー';
    $this->assertEquals($expected, NeologdNormalizer::convertLongVowelSymbols($input));
  }

  /**
   * Tests convertDeleteTildeSymbols method.
   */
  public function testConvertDeleteTildeSymbols() {
    $input = '～∼∾〜〰～';
    $expected = '';
    $this->assertEmpty($expected, NeologdNormalizer::convertDeleteTildeSymbols($input));
  }

  /**
   * Tests convertHalfWidthSymbols method.
   */
  public function testConvertHalfWidthSymbols() {
    $input = '／！”＃＄％＆’（）＊＋，−．／：；＜＞？＠［￥］＾＿｀｛｜｝';
    $expected = '/!”#$%&’()*+,-./:;<>?@[¥]^_`{|}';
    $this->assertEquals($expected, NeologdNormalizer::convertHalfWidthSymbols($input));
  }

  /**
   * Tests convertHalfWidthSymbolsToFullWidth method.
   */
  public function testConvertHalfWidthSymbolsToFullWidth() {
    $input = '｡､･=｢｣';
    $expected = '。、・＝「」';
    $this->assertEquals($expected, NeologdNormalizer::convertHalfWidthSymbolsToFullWidth($input));
  }

  /**
   * Tests convertFullWidthSpaceToHalfWidth method.
   */
  public function testConvertFullWidthSpaceToHalfWidth() {
    $input = 'Hello　World';
    $expected = 'Hello World';
    $this->assertEquals($expected, NeologdNormalizer::convertFullWidthSpaceToHalfWidth($input));
  }

  /**
   * Tests convertMultipleSpaces method.
   */
  public function testConvertMultipleSpaces() {
    $input = 'Hello    World';
    $expected = 'Hello World';
    $this->assertEquals($expected, NeologdNormalizer::convertMultipleSpaces($input));
  }

  /**
   * Tests convertUnnecessaryHalfWidthSpaces method.
   *
   * @dataProvider provideUnnecessaryHalfWidthSpaces
   */
  public function testConvertUnnecessaryHalfWidthSpaces($input, $expected) {
    $this->assertEquals($expected, NeologdNormalizer::convertUnnecessaryHalfWidthSpaces($input));
  }

  /**
   * Data provider for testConvertUnnecessaryHalfWidthSpaces.
   */
  public function provideUnnecessaryHalfWidthSpaces() {
    return [
      ['検索 エンジン 自作 入門 を 買い ました ！！！', '検索エンジン自作入門を買いました！！！'],
      ['私は Experience Builder Initiative に 期待 している', '私は Experience Builder Initiative に期待している'],
    ];
  }

  /**
   * Tests convertTrimSpaceJapaneseAlpha method.
   *
   * @dataProvider provideTrimSpaceJapaneseAlpha
   */
  public function testConvertTrimSpaceJapaneseAlpha($input, $expected) {
    $this->assertEquals($expected, NeologdNormalizer::convertTrimSpaceJapaneseAlpha($input));
  }

  /**
   * Data provider for testConvertTrimSpaceJapaneseAlpha.
   */
  public function provideTrimSpaceJapaneseAlpha() {
    return [
      ['アルゴリズム C', 'アルゴリズムC'],
      ['Algorithm C', 'Algorithm C'],
      ['Drupal は CMS', 'DrupalはCMS'],
      ['私は！ Drupal （ CMS ）が好きです。', '私は！Drupal（CMS）が好きです。'],
      ['私は Hermès のバッグを持っている', '私はHermèsのバッグを持っている'],
      ['Ｄｒｕｐａｌ 十', 'Ｄｒｕｐａｌ十'],
      ['2025 年 の トレンド Drupal', '2025年のトレンドDrupal'],
    ];
  }

}
